@@ -24,13 +24,18 @@
 #include <string.h>
 #include <stdbool.h>
 #include <stdlib.h>
-#include <cli_classic.h>
+#include <stdint.h>
+#include <cli_getopt.h>
 #include "flash.h"
 #include "flashchips.h"
 #include "fmap.h"
 #include "programmer.h"
 #include "libflashrom.h"
 
+#if CONFIG_RPMC_ENABLED == 1
+#include "rpmc.h"
+#endif /* CONFIG_RPMC_ENABLED */
+
 enum {
 	OPTION_IFD = 0x0100,
 	OPTION_FMAP,
@@ -45,15 +50,23 @@ enum {
 	OPTION_WP_DISABLE,
 	OPTION_WP_LIST,
 	OPTION_PROGRESS,
+	OPTION_SACRIFICE_RATIO,
+#if CONFIG_RPMC_ENABLED == 1
+	OPTION_RPMC_READ_DATA,
+	OPTION_RPMC_WRITE_ROOT_KEY,
+	OPTION_RPMC_UPDATE_HMAC_KEY,
+	OPTION_RPMC_INCREMENT_COUNTER,
+	OPTION_RPMC_GET_COUNTER,
+	OPTION_RPMC_COUNTER_ADDRESS,
+	OPTION_RPMC_KEY_DATA,
+	OPTION_RPMC_KEY_FILE,
+#endif /* CONFIG_RPMC_ENABLED */
 };
 
 struct cli_options {
 	bool read_it, extract_it, write_it, erase_it, verify_it;
 	bool dont_verify_it, dont_verify_all;
 	bool list_supported;
-#if CONFIG_PRINT_WIKI == 1
-	bool list_supported_wiki;
-#endif
 	char *filename;
 
 	const struct programmer_entry *prog;
@@ -76,14 +89,23 @@ struct cli_options {
 	char *logfile;
 	char *referencefile;
 	const char *chip_to_probe;
+	int sacrifice_ratio;
+
+#if CONFIG_RPMC_ENABLED == 1
+	bool rpmc_read_data;
+	bool rpmc_write_root_key;
+	bool rpmc_update_hmac_key;
+	bool rpmc_increment_counter;
+	bool rpmc_get_counter;
+	unsigned int rpmc_counter_address;
+	uint32_t rpmc_key_data, rpmc_previous_counter_value;
+	const char *rpmc_root_key_file;
+#endif /* CONFIG_RPMC_ENABLED */
 };
 
 static void cli_classic_usage(const char *name)
 {
 	printf("Usage: %s [-h|-R|-L|"
-#if CONFIG_PRINT_WIKI == 1
-	       "-z|"
-#endif
 	       "\n\t-p <programmername>[:<parameters>] [-c <chipname>]\n"
 	       "\t\t(--flash-name|--flash-size|\n"
 	       "\t\t [-E|-x|(-r|-w|-v) <file>]\n"
@@ -124,17 +146,36 @@ static void cli_classic_usage(const char *name)
 	       " -o | --output <logfile>            log output to <logfile>\n"
 	       "      --flash-contents <ref-file>   assume flash contents to be <ref-file>\n"
 	       " -L | --list-supported              print supported devices\n"
-#if CONFIG_PRINT_WIKI == 1
-	       " -z | --list-supported-wiki         print supported devices in wiki syntax\n"
-#endif
 	       "      --progress                    show progress percentage on the standard output\n"
+	       "      --sacrifice-ratio <ratio>     Fraction (as a percentage, 0-50) of an erase block\n"
+	       "                                    that may be erased even if unmodified. Larger values\n"
+	       "				    may complete programming faster, but may also hurt\n"
+	       "				    chip longevity by erasing cells unnecessarily.\n"
+	       "				    Default is 0, tradeoff is the speed of programming\n"
+	       "                                    operation VS the longevity of the chip. Default is\n"
+	       "                                    longevity.\n"
+	       "                                    DANGEROUS! It wears your chip faster!\n"
+#if CONFIG_RPMC_ENABLED == 1
+	       "RPMC COMMANDS\n"
+	       "      --get-rpmc-status             read the extended status\n"
+	       "      --write-root-key              write the root key register\n"
+	       "      --update-hmac-key             update the hmac key register\n"
+	       "      --increment-counter <current>\n"
+	       "                                    increment rpmc counter\n"
+	       "      --get-counter                 get rpmc counter\n"
+	       "RPMC OPTIONS\n"
+	       "      --counter-address <address>   counter address (default: 0)\n"
+	       "      --rpmc-root-key <keyfile>     rpmc root key file\n"
+	       "      --key-data <value>            hex number to use as key data (default: 0)\n"
+#endif /* CONFIG_RPMC_ENABLED */
+	       "PROGRAMMER SELECTION OPTIONS\n"
 	       " -p | --programmer <name>[:<param>] specify the programmer device. One of\n");
 	list_programmers_linebreak(4, 80, 0);
-	printf(".\n\nYou can specify one of -h, -R, -L, "
-#if CONFIG_PRINT_WIKI == 1
-	         "-z, "
-#endif
-	         "-E, -r, -w, -v or no operation.\n"
+	printf(".\n\nYou can specify one of -h, -R, -L, -E, -r, -w, -v"
+#if CONFIG_RPMC_ENABLED == 1
+		 ", a RPMC command"
+#endif /* CONFIG_RPMC_ENABLED */
+		 " or no operation.\n"
 	       "If no operation is specified, flashrom will only probe for flash chips.\n");
 }
 
@@ -407,6 +448,110 @@ static int wp_cli(
 	return 0;
 }
 
+#if CONFIG_RPMC_ENABLED == 1
+static int rpmc_cli(struct flashctx *flash,
+		    const char *const key_file,
+		    const uint32_t key_data,
+		    const unsigned int counter_address,
+		    const uint32_t previous_counter,
+		    const bool op_read_data,
+		    const bool op_write_root_key,
+		    const bool op_update_hmac_key,
+		    const bool op_increment_counter,
+		    const bool op_get_counter)
+{
+	if (op_write_root_key) {
+		enum rpmc_result result = rpmc_write_root_key(flash, key_file, counter_address);
+		if (result != RPMC_SUCCESS) {
+			msg_gerr("Failed to write root key\n%s", rpmc_describe_result(result));
+			return 1;
+		}
+
+		msg_ginfo("Successfully wrote new root key for counter %u.\n", counter_address);
+	}
+
+	if (op_update_hmac_key) {
+		enum rpmc_result result = rpmc_update_hmac_key(flash,
+							       key_file,
+							       key_data,
+							       counter_address);
+		if (result != RPMC_SUCCESS) {
+			msg_gerr("Failed to update hmac key\n%s", rpmc_describe_result(result));
+			return 1;
+		}
+
+		msg_ginfo("Successfully updated hmac key to 0x%08x for counter %u.\n",
+			  key_data,
+			  counter_address);
+	}
+
+	if (op_increment_counter) {
+		enum rpmc_result result = rpmc_increment_counter(flash,
+								 key_file,
+								 key_data,
+								 counter_address,
+								 previous_counter);
+		if (result != RPMC_SUCCESS) {
+			msg_gerr("Failed to increment the counter\n%s", rpmc_describe_result(result));
+			return 1;
+		}
+
+		msg_ginfo("Successfully incremented counter %u.\n", counter_address);
+	}
+
+	if (op_get_counter) {
+		uint32_t counter_value;
+		enum rpmc_result result = rpmc_get_monotonic_counter(flash,
+								     key_file,
+								     key_data,
+								     counter_address,
+								     &counter_value);
+		if (result != RPMC_SUCCESS) {
+			msg_gerr("Failed to get the counter value\n%s", rpmc_describe_result(result));
+			return 1;
+		}
+
+		msg_ginfo("Returned counter value %u for counter %u\n",	counter_value, counter_address);
+	}
+
+	if (op_read_data) {
+		struct rpmc_status_register status;
+		enum rpmc_result result = rpmc_read_data(flash, &status);
+		if (result != RPMC_SUCCESS) {
+			msg_gerr("Failed to read read rpmc data\n%s", rpmc_describe_result(result));
+			return 1;
+		}
+
+		msg_ginfo("Reading rpmc data returned:\n");
+
+		char bin_buffer[9];
+		uint8_t status_bits = status.status;
+		for (int i = 7; i >= 0; i--){
+			bin_buffer[i] = '0' + (status_bits & 1);
+			status_bits = status_bits >> 1;
+		}
+		bin_buffer[8] = '\0';
+		msg_ginfo("Extended Status: 0b%s\n", bin_buffer);
+
+		msg_ginfo("Tag:\n");
+		for (size_t i = 0; i < RPMC_TAG_LENGTH; i++){
+			msg_ginfo("0x%02x ", status.tag[i]);
+		}
+		msg_ginfo("\n");
+
+		msg_ginfo("Counter: %u\n", status.counter_data);
+
+		msg_ginfo("Signature:\n");
+		for (size_t i = 0; i < RPMC_SIGNATURE_LENGTH; i++){
+			msg_ginfo("0x%02x ", status.signature[i]);
+		}
+		msg_ginfo("\n");
+	}
+
+	return 0;
+}
+#endif /* CONFIG_RPMC_ENABLED */
+
 /**
  * @brief Reads content to buffer from one or more files.
  *
@@ -754,15 +899,6 @@ static void parse_options(int argc, char **argv, const char *optstring,
 			cli_classic_validate_singleop(&operation_specified);
 			options->list_supported = true;
 			break;
-		case 'z':
-#if CONFIG_PRINT_WIKI == 1
-			cli_classic_validate_singleop(&operation_specified);
-			options->list_supported_wiki = true;
-#else
-			cli_classic_abort_usage("Error: Wiki output was not "
-					"compiled in. Aborting.\n");
-#endif
-			break;
 		case 'p':
 			if (options->prog != NULL) {
 				cli_classic_abort_usage("Error: --programmer specified "
@@ -831,6 +967,37 @@ static void parse_options(int argc, char **argv, const char *optstring,
 		case OPTION_PROGRESS:
 			options->show_progress = true;
 			break;
+		case OPTION_SACRIFICE_RATIO:
+			/* It is okay to convert invalid input to 0. */
+			options->sacrifice_ratio = atoi(optarg);
+			break;
+#if CONFIG_RPMC_ENABLED == 1
+		case OPTION_RPMC_READ_DATA:
+			options->rpmc_read_data = true;
+			break;
+		case OPTION_RPMC_WRITE_ROOT_KEY:
+			options->rpmc_write_root_key = true;
+			break;
+		case OPTION_RPMC_UPDATE_HMAC_KEY:
+			options->rpmc_update_hmac_key = true;
+			break;
+		case OPTION_RPMC_INCREMENT_COUNTER:
+			options->rpmc_increment_counter = true;
+			options->rpmc_previous_counter_value = strtoumax(optarg, NULL, 10);
+			break;
+		case OPTION_RPMC_GET_COUNTER:
+			options->rpmc_get_counter = true;
+			break;
+		case OPTION_RPMC_COUNTER_ADDRESS:
+			options->rpmc_counter_address = strtoumax(optarg, NULL, 10);
+			break;
+		case OPTION_RPMC_KEY_DATA:
+			options->rpmc_key_data = strtoumax(optarg, NULL, 16);
+			break;
+		case OPTION_RPMC_KEY_FILE:
+			options->rpmc_root_key_file = strdup(optarg);
+			break;
+#endif /* CONFIG_RPMC_ENABLED */
 		default:
 			cli_classic_abort_usage(NULL);
 			break;
@@ -895,12 +1062,22 @@ int main(int argc, char *argv[])
 		{"wp-enable",		0, NULL, OPTION_WP_ENABLE},
 		{"wp-disable",		0, NULL, OPTION_WP_DISABLE},
 		{"list-supported",	0, NULL, 'L'},
-		{"list-supported-wiki",	0, NULL, 'z'},
 		{"programmer",		1, NULL, 'p'},
 		{"help",		0, NULL, 'h'},
 		{"version",		0, NULL, 'R'},
 		{"output",		1, NULL, 'o'},
 		{"progress",		0, NULL, OPTION_PROGRESS},
+		{"sacrifice-ratio",	1, NULL, OPTION_SACRIFICE_RATIO},
+#if CONFIG_RPMC_ENABLED == 1
+		{"get-rpmc-status",	0, NULL, OPTION_RPMC_READ_DATA},
+		{"write-root-key",	0, NULL, OPTION_RPMC_WRITE_ROOT_KEY},
+		{"update-hmac-key",	0, NULL, OPTION_RPMC_UPDATE_HMAC_KEY},
+		{"increment-counter",	1, NULL, OPTION_RPMC_INCREMENT_COUNTER},
+		{"get-counter",		0, NULL, OPTION_RPMC_GET_COUNTER},
+		{"counter-address",	1, NULL, OPTION_RPMC_COUNTER_ADDRESS},
+		{"key-data",		1, NULL, OPTION_RPMC_KEY_DATA},
+		{"rpmc-root-key",	1, NULL, OPTION_RPMC_KEY_FILE},
+#endif /* CONFIG_RPMC_ENABLED */
 		{NULL,			0, NULL, 0},
 	};
 
@@ -934,13 +1111,6 @@ int main(int argc, char *argv[])
 	if (options.logfile && open_logfile(options.logfile))
 		cli_classic_abort_usage(NULL);
 
-#if CONFIG_PRINT_WIKI == 1
-	if (options.list_supported_wiki) {
-		print_supported_wiki();
-		goto out;
-	}
-#endif
-
 	if (options.list_supported) {
 		if (print_supported())
 			ret = 1;
@@ -1089,9 +1259,9 @@ int main(int argc, char *argv[])
 
 	fill_flash = &flashes[0];
 
-	unsigned int progress_user_data[FLASHROM_PROGRESS_NR];
+	struct cli_progress cli_progress = {0};
 	struct flashrom_progress progress_state = {
-		 .user_data = progress_user_data
+		 .user_data = &cli_progress
 	};
 	if (options.show_progress)
 		flashrom_set_progress_callback(fill_flash, &flashrom_progress_cb, &progress_state);
@@ -1118,9 +1288,17 @@ int main(int argc, char *argv[])
 		options.set_wp_range || options.set_wp_region || options.enable_wp ||
 		options.disable_wp || options.print_wp_status || options.print_wp_ranges;
 
+	const bool any_rpmc_op =
+#if CONFIG_RPMC_ENABLED == 1
+		options.rpmc_read_data || options.rpmc_write_root_key || options.rpmc_update_hmac_key ||
+		options.rpmc_increment_counter || options.rpmc_get_counter;
+#else
+		false;
+#endif /* CONFIG_RPMC_ENABLED */
+
 	const bool any_op = options.read_it || options.write_it || options.verify_it ||
 		options.erase_it || options.flash_name || options.flash_size ||
-		options.extract_it || any_wp_op;
+		options.extract_it || any_wp_op || any_rpmc_op;
 
 	if (!any_op) {
 		msg_ginfo("No operations were specified.\n");
@@ -1154,6 +1332,14 @@ int main(int argc, char *argv[])
 		goto out_shutdown;
 	}
 
+	if (options.sacrifice_ratio) {
+		if (options.sacrifice_ratio < 0 || options.sacrifice_ratio > 50) {
+			msg_ginfo("Invalid input of sacrifice ratio, valid 0-50. Fallback to default value 0.\n");
+			options.sacrifice_ratio = 0;
+		}
+		fill_flash->sacrifice_ratio = options.sacrifice_ratio;
+	}
+
 	if (options.ifd && (flashrom_layout_read_from_ifd(&options.layout, fill_flash, NULL, 0) ||
 			   process_include_args(options.layout, options.include_args))) {
 		ret = 1;
@@ -1256,6 +1442,21 @@ int main(int argc, char *argv[])
 	else if (options.verify_it)
 		ret = do_verify(fill_flash, options.filename);
 
+#if CONFIG_RPMC_ENABLED == 1
+	if (any_rpmc_op && ret == 0) {
+		ret = rpmc_cli(fill_flash,
+			       options.rpmc_root_key_file,
+			       options.rpmc_key_data,
+			       options.rpmc_counter_address,
+			       options.rpmc_previous_counter_value,
+			       options.rpmc_read_data,
+			       options.rpmc_write_root_key,
+			       options.rpmc_update_hmac_key,
+			       options.rpmc_increment_counter,
+			       options.rpmc_get_counter);
+	}
+#endif /* CONFIG_RPMC_ENABLED */
+
 out_release:
 	flashrom_layout_release(options.layout);
 out_shutdown:
